from chroma_feedback import request


def test_get() -> None:
	response = request.get('https://jsonplaceholder.typicode.com/posts',
	{
		'Content-Type': 'application/json'
	})

	assert response.status_code == 200


def test_post() -> None:
	response = request.post('https://jsonplaceholder.typicode.com/posts', headers =
	{
		'Content-Type': 'application/json'
	})

	assert response.status_code == 201


def test_parse_json() -> None:
	response = request.post('https://jsonplaceholder.typicode.com/posts', headers =
	{
		'Content-Type': 'application/json'
	})
	data = request.parse_json(response)

	assert data['id']


def test_create_basic_auth_headers() -> None:
	headers = request.create_basic_auth_headers('abc', '12345')

	assert headers['Accept'] == 'application/json'
	assert headers['Authorization'] == 'Basic YWJjOjEyMzQ1'


def test_create_bearer_auth_headers() -> None:
	headers = request.create_bearer_auth_headers('12345')

	assert headers['Accept'] == 'application/json'
	assert headers['Authorization'] == 'Bearer 12345'



import sys
from typing import Any

from chroma_feedback import logger, wording

API = None


def get_api() -> Any:
	global API

	if not API:
		API = api_factory()
	return API


def api_factory() -> Any:
	try:
		from busylight.lights import NoLightsFound, LightUnavailable
		from busylight.lights.muteme import MuteMe_Mini as api

		try:
			api.first_light().release()
		except (NoLightsFound, LightUnavailable):
			logger.error(wording.get('connection_not_found').format('muteme.muteme_mini') + wording.get('exclamation_mark'))
			sys.exit()
		return api
	except ImportError:
		logger.error(wording.get('package_not_found').format('busylight-for-humans') + wording.get('exclamation_mark'))
		sys.exit()



from logging import DEBUG, ERROR, INFO, Logger, WARNING, basicConfig, getLogger
from typing import Dict

from chroma_feedback.typing import LogLevel


def init(log_level : LogLevel) -> None:
	basicConfig(format = '%(message)s')
	get_package_logger().setLevel(get_log_levels()[log_level])


def get_package_logger() -> Logger:
	return getLogger('chroma_feedback')


def debug(message : str) -> None:
	get_package_logger().debug(message)


def info(message : str) -> None:
	get_package_logger().info(message)


def warn(message : str) -> None:
	get_package_logger().warning(message)


def error(message : str) -> None:
	get_package_logger().error(message)


def get_log_levels() -> Dict[LogLevel, int]:
	return\
	{
		'error': ERROR,
		'warn': WARNING,
		'info': INFO,
		'debug': DEBUG
	}



from chroma_feedback.typing import Producer, Status


def normalize_data(slug : str, url : str, status : int) -> Producer:
	return\
	{
		'name': 'uptimerobot',
		'slug': slug,
		'url': url,
		'status': normalize_status(status)
	}


def normalize_status(status : int) -> Status:
	if status == 0:
		return 'skipped'
	if status == 1:
		return 'started'
	if status == 9:
		return 'failed'
	return 'passed'



import sys

from chroma_feedback import helper


def test_is_root() -> None:
	assert helper.is_root() is not None


def test_is_linux() -> None:
	assert helper.is_linux() is not None


def test_is_macos() -> None:
	assert helper.is_macos() is not None


def test_is_windows() -> None:
	assert helper.is_linux() is not None


def test_parse_slug() -> None:
	assert 'workspace' in helper.parse_slug('henryruhs')
	assert 'project' not in helper.parse_slug('henryruhs')
	assert 'workspace' in helper.parse_slug('henryruhs/chroma-feedback')
	assert 'project' in helper.parse_slug('henryruhs/chroma-feedback')
	assert 'workspace' not in helper.parse_slug(None)
	assert 'project' not in helper.parse_slug(None)


def test_create_description() -> None:
	assert helper.create_description('name', 'selector') == 'name [selector]'
	assert helper.create_description('name', None) == 'name'
	assert helper.create_description(None, 'selector') == 'selector'


def test_to_lower_case() -> None:
	assert helper.to_lower_case(None) == 'none'
	assert helper.to_lower_case('SUCCESS') == 'success'


def test_has_argument() -> None:
	sys.argv.clear()
	sys.argv.append('--valid')

	assert helper.has_argument('--valid') is True


def test_has_argument_invalid() -> None:
	sys.argv.clear()
	sys.argv.append('--invalid')

	assert helper.has_argument('--valid') is False


def test_get_first() -> None:
	assert helper.get_first(
	[
		1,
		2,
		3
	]) == 1


def test_get_last() -> None:
	assert helper.get_last(
	[
		1,
		2,
		3
	]) == 3


def test_remove_duplicate() -> None:
	assert helper.remove_duplicate(
	[
		1,
		1,
		2,
		2,
		3,
		3
	]) ==\
	[
		1,
		2,
		3
	]



from .core import process

ALL = \
{
	'appveyor': 'chroma_feedback.producer.appveyor.appveyor',
	'atlassian.bamboo': 'chroma_feedback.producer.atlassian.bamboo',
	'atlassian.bitbucket': 'chroma_feedback.producer.atlassian.bitbucket',
	'buddy': 'chroma_feedback.producer.buddy.buddy',
	'circle': 'chroma_feedback.producer.circle.circle',
	'cloudbees.codeship': 'chroma_feedback.producer.cloudbees.codeship',
	'custom': 'chroma_feedback.producer.custom.custom',
	'datadog': 'chroma_feedback.producer.datadog.datadog',
	'gitlab': 'chroma_feedback.producer.gitlab.gitlab',
	'heroku': 'chroma_feedback.producer.heroku.heroku',
	'jenkins': 'chroma_feedback.producer.jenkins.jenkins',
	'jetbrains.teamcity': 'chroma_feedback.producer.jetbrains.teamcity',
	'microsoft.azure': 'chroma_feedback.producer.microsoft.azure',
	'microsoft.github': 'chroma_feedback.producer.microsoft.github',
	'netlify': 'chroma_feedback.producer.netlify.netlify',
	'travis': 'chroma_feedback.producer.travis.travis',
	'uptimerobot': 'chroma_feedback.producer.uptimerobot.uptimerobot',
	'vercel': 'chroma_feedback.producer.vercel.vercel'
}



import atexit
from typing import Any, List

from chroma_feedback import color, helper, reporter
from chroma_feedback.typing import Color, Consumer, ProducerReport, Status
from .api import get_api

DEVICES = None


def get_devices() -> Any:
	global DEVICES

	if not DEVICES:
		DEVICES = get_api().find_supported_devices()
	return DEVICES


def process_devices(devices : Any, producer_report : List[ProducerReport]) -> List[Consumer]:
	result : List[Consumer] = []
	status : Status = reporter.resolve_report_status(producer_report)

	for device in devices:
		set_device(device, color.get_by_status(status))
		if helper.has_argument('-b') or helper.has_argument('--background-run'):
			register_reset_device(device)
		result.append(
		{
			'name': 'gigabyte.rgb_fusion2',
			'type': 'device',
			'description': device.description,
			'status': status
		})
	return result


def set_device(device : Any, color_config : Color) -> None:
	with device.connect():
		device.set_color('sync', 'fixed',
		[
			tuple(color_config['rgb'])
		])


def register_reset_device(device : Any) -> None:
	atexit.register(lambda: set_device(device, color.get_reset()))



from typing import Dict, List, Literal, Optional, TypedDict

Status = Literal\
[
	'skipped',
	'passed',
	'started',
	'errored',
	'warned',
	'failed'
]
ProducerName = Literal\
[
	'appveyor',
	'atlassian.bamboo',
	'atlassian.bitbucket',
	'buddy',
	'circle',
	'cloudbees.codeship',
	'custom',
	'datadog',
	'gitlab',
	'heroku',
	'jenkins',
	'jetbrains.teamcity',
	'microsoft.azure',
	'microsoft.github',
	'netlify',
	'travis',
	'uptimerobot',
	'vercel'
]
ConsumerName = Literal\
[
	'agile_innovative.blinkstick',
	'compulab.fit_statusb',
	'elgato.streamdeck',
	'embrava.blynclight',
	'embrava.blynclight_mini',
	'embrava.blynclight_plus',
	'gigabyte.rgb_fusion2',
	'kuando.busylight_alpha',
	'kuando.busylight_omega',
	'lifx',
	'luxafor.flag',
	'luxafor.mute',
	'luxafor.orb',
	'magic.hue',
	'muteme',
	'muteme.muteme_mini',
	'mutesync',
	'nanoleaf',
	'philips.hue',
	'plantronics.status_indicator',
	'razer.chroma',
	'signify.wiz',
	'thingm.blink1',
	'xiaomi.yeelight'
]
ConsumerType = Literal\
[
	'light',
	'device',
	'group'
]
LogLevel = Literal\
[
	'error',
	'warn',
	'info',
	'debug'
]
Headers = Dict[str, str]


class Producer(TypedDict):
	name : ProducerName
	slug : str
	url : Optional[str]
	status : Status


class Consumer(TypedDict):
	name : ConsumerName
	type : ConsumerType
	description : str
	status : Status


class ProducerReport(TypedDict):
	name : ProducerName
	symbol : str
	message : str
	url : Optional[str]
	status : Status


class ConsumerReport(TypedDict):
	name : ConsumerName
	symbol : str
	message : str
	status : Status


class Color(TypedDict):
	rgb : List[int]
	hue : int
	saturation : List[int]
	brightness : List[int]
	kelvin : int



import atexit
import copy
from typing import Any, List

from PyQt6 import QtCore
from PyQt6.QtCore import QRect, Qt
from PyQt6.QtGui import QColor, QFont, QPainter, QPen, QPixmap, QTransform

from chroma_feedback import color, helper, reporter
from chroma_feedback.typing import Consumer, ProducerReport, Status
from .api import get_api

DEVICES = None


def get_devices() -> Any:
	global DEVICES

	if not DEVICES:
		DEVICES = get_api().enumerate()
	return DEVICES


def filter_devices(devices : Any, device_ids : List[str]) -> Any:
	if device_ids:
		for device in copy.copy(devices):
			if device.id() not in device_ids:
				devices.remove(device)
	return devices


def process_devices(devices : Any, producer_report : List[ProducerReport]) -> List[Consumer]:
	result : List[Consumer] = []
	status : Status = reporter.resolve_report_status(producer_report)

	for device in devices:
		if set_device(device, producer_report):
			register_close_device(device)
			if helper.has_argument('-b') or helper.has_argument('--background-run'):
				register_reset_device(device)
			result.append(
			{
				'name': 'elgato.streamdeck',
				'type': 'device',
				'description': helper.create_description(device.deck_type(), device.id()),
				'status': status
			})
	return result


def set_device(device : Any, producer_report : List[ProducerReport]) -> bool:
	device.open()

	for index in range(device.key_count()):
		if device.KEY_PIXEL_HEIGHT > 0 and device.KEY_PIXEL_WIDTH > 0:
			if index < len(producer_report):
				device.set_key_image(index, create_image(device, producer_report[index]))
			else:
				device.set_key_image(index, None)

	return device.is_open()


def register_close_device(device : Any) -> None:
	atexit.register(lambda: device.close())


def register_reset_device(device : Any) -> None:
	atexit.register(lambda: device.reset())


def create_image(device : Any, report : ProducerReport) -> bytes:
	color_config = color.get_by_status(report['status'])
	image_config = device.key_image_format()
	transform = create_transform(image_config)
	font_size = image_config['size'][0] / 8
	pixmap = QPixmap(*image_config['size'])
	pixmap.fill(Qt.GlobalColor.transparent)
	painter = QPainter(pixmap)
	painter.setFont(create_font(int(font_size * 2.75), QFont.Weight.Bold))
	painter.setPen(QPen(QColor(*color_config['rgb'])))
	painter.drawText(QRect(0, int(font_size * 1.5), *image_config['size']), Qt.AlignmentFlag.AlignCenter, report['symbol'])
	painter.setFont(create_font(int(font_size * 0.875), QFont.Weight.Bold))
	painter.setPen(QPen(Qt.GlobalColor.white))
	painter.drawText(QRect(0, int(font_size * -1.875), *image_config['size']), Qt.AlignmentFlag.AlignCenter, report['name'].split('.')[-1].upper())
	painter.end()
	return pixmap_to_bytes(pixmap.transformed(transform), image_config)


def create_transform(image_config : Any) -> QTransform:
	transform = QTransform()

	if image_config['flip'][0]:
		transform.scale(-1, 1)
	if image_config['flip'][1]:
		transform.scale(1, -1)
	if image_config['rotation']:
		transform.rotate(image_config['rotation'])
	return transform


def create_font(font_size : int, font_weight : int) -> QFont:
	font = QFont()
	font.setPointSize(font_size)
	font.setWeight(font_weight)
	return font


def pixmap_to_bytes(pixmap : QPixmap, image_config : Any) -> bytes:
	byte_array = QtCore.QByteArray()
	buffer = QtCore.QBuffer(byte_array)
	buffer.open(QtCore.QIODevice.OpenModeFlag.WriteOnly)
	pixmap.save(buffer, image_config['format'])
	return byte_array.data()



import sys
from argparse import ArgumentParser
from typing import List

from chroma_feedback import helper, logger, wording
from chroma_feedback.typing import Consumer, ProducerReport
from .device import filter_devices, get_devices, process_devices

ARGS = None


def support() -> bool:
	return helper.is_linux()


def init(program : ArgumentParser) -> None:
	global ARGS

	if not ARGS:
		program.add_argument('--razer-chroma-device-serial', action = 'append')
	ARGS = helper.get_first(program.parse_known_args())


def run(producer_report : List[ProducerReport]) -> List[Consumer]:
	devices = filter_devices(get_devices(), ARGS.razer_chroma_device_serial)

	if not devices:
		logger.error(wording.get('device_not_found') + wording.get('exclamation_mark'))
		sys.exit()
	return process_devices(devices, producer_report)



from chroma_feedback import reporter


def test_create_producer_report_passed() -> None:
	producer_report = reporter.create_producer_report(
	[
		{
			'name': 'microsoft.github',
			'slug': 'henryruhs/chroma-feedback',
			'url': 'https://github.com/henryruhs/chroma-feedback',
			'status': 'passed'
		}
	])

	assert producer_report[0]['symbol']
	assert producer_report[0]['message'] == 'Status of henryruhs/chroma-feedback on microsoft.github is passed'
	assert producer_report[0]['url'] == 'https://github.com/henryruhs/chroma-feedback'
	assert producer_report[0]['status'] == 'passed'


def test_create_producer_report_started() -> None:
	producer_report = reporter.create_producer_report(
	[
		{
			'name': 'microsoft.github',
			'slug': 'henryruhs/chroma-feedback',
			'url': 'https://github.com/henryruhs/chroma-feedback',
			'status': 'started'
		}
	])

	assert producer_report[0]['symbol']
	assert producer_report[0]['message'] == 'Status of henryruhs/chroma-feedback on microsoft.github is started'
	assert producer_report[0]['url'] == 'https://github.com/henryruhs/chroma-feedback'
	assert producer_report[0]['status'] == 'started'


def test_create_producer_report_errored() -> None:
	producer_report = reporter.create_producer_report(
	[
		{
			'name': 'microsoft.github',
			'slug': 'henryruhs/chroma-feedback',
			'url': 'https://github.com/henryruhs/chroma-feedback',
			'status': 'errored'
		}
	])

	assert producer_report[0]['symbol']
	assert producer_report[0]['message'] == 'Status of henryruhs/chroma-feedback on microsoft.github is errored'
	assert producer_report[0]['url'] == 'https://github.com/henryruhs/chroma-feedback'
	assert producer_report[0]['status'] == 'errored'


def test_create_producer_report_warned() -> None:
	producer_report = reporter.create_producer_report(
	[
		{
			'name': 'microsoft.github',
			'slug': 'henryruhs/chroma-feedback',
			'url': 'https://github.com/henryruhs/chroma-feedback',
			'status': 'warned'
		}
	])

	assert producer_report[0]['symbol']
	assert producer_report[0]['message'] == 'Status of henryruhs/chroma-feedback on microsoft.github is warned'
	assert producer_report[0]['url'] == 'https://github.com/henryruhs/chroma-feedback'
	assert producer_report[0]['status'] == 'warned'


def test_create_producer_report_failed() -> None:
	producer_report = reporter.create_producer_report(
	[
		{
			'name': 'microsoft.github',
			'slug': 'henryruhs/chroma-feedback',
			'url': 'https://github.com/henryruhs/chroma-feedback',
			'status': 'failed'
		}
	])

	assert producer_report[0]['symbol']
	assert producer_report[0]['message'] == 'Status of henryruhs/chroma-feedback on microsoft.github is failed'
	assert producer_report[0]['status'] == 'failed'


def test_create_consumer_report_passed() -> None:
	consumer_report = reporter.create_consumer_report(
	[
		{
			'name': 'razer.chroma',
			'type': 'device',
			'description': 'Razer Huntsman Elite',
			'status': 'passed'
		}
	])

	assert consumer_report[0]['symbol']
	assert consumer_report[0]['message'] == 'Set status of Razer Huntsman Elite to passed'
	assert consumer_report[0]['status'] == 'passed'


def test_create_consumer_report_started() -> None:
	consumer_report = reporter.create_consumer_report(
	[
		{
			'name': 'razer.chroma',
			'type': 'device',
			'description': 'Razer Huntsman Elite',
			'status': 'started'
		}
	])

	assert consumer_report[0]['symbol']
	assert consumer_report[0]['message'] == 'Set status of Razer Huntsman Elite to started'
	assert consumer_report[0]['status'] == 'started'


def test_create_consumer_report_errored() -> None:
	consumer_report = reporter.create_consumer_report(
	[
		{
			'name': 'razer.chroma',
			'type': 'device',
			'description': 'Razer Huntsman Elite',
			'status': 'errored'
		}
	])

	assert consumer_report[0]['symbol']
	assert consumer_report[0]['message'] == 'Set status of Razer Huntsman Elite to errored'
	assert consumer_report[0]['status'] == 'errored'


def test_create_consumer_report_warned() -> None:
	consumer_report = reporter.create_consumer_report(
	[
		{
			'name': 'razer.chroma',
			'type': 'device',
			'description': 'Razer Huntsman Elite',
			'status': 'warned'
		}
	])

	assert consumer_report[0]['symbol']
	assert consumer_report[0]['message'] == 'Set status of Razer Huntsman Elite to warned'
	assert consumer_report[0]['status'] == 'warned'


def test_create_consumer_report_failed() -> None:
	consumer_report = reporter.create_consumer_report(
	[
		{
			'name': 'razer.chroma',
			'type': 'device',
			'description': 'Razer Huntsman Elite',
			'status': 'failed'
		}
	])

	assert consumer_report[0]['symbol']
	assert consumer_report[0]['message'] == 'Set status of Razer Huntsman Elite to failed'
	assert consumer_report[0]['status'] == 'failed'


def test_resolve_report_status_started() -> None:
	assert reporter.resolve_report_status(
	[
		{
			'name': 'microsoft.github',
			'symbol': None,
			'message': None,
			'url': 'https://github.com/henryruhs/chroma-feedback/actions/runs/1',
			'status': 'started'
		},
		{
			'name': 'microsoft.github',
			'symbol': None,
			'message': None,
			'url': 'https://github.com/henryruhs/chroma-feedback/actions/runs/1',
			'status': 'passed'
		}
	]) == 'started'


def test_resolve_report_status_errored() -> None:
	assert reporter.resolve_report_status(
	[
		{
			'name': 'microsoft.github',
			'symbol': None,
			'message': None,
			'url': 'https://github.com/henryruhs/chroma-feedback/actions/runs/1',
			'status': 'errored'
		},
		{
			'name': 'microsoft.github',
			'symbol': None,
			'message': None,
			'url': 'https://github.com/henryruhs/chroma-feedback/actions/runs/1',
			'status': 'started'
		},
		{
			'name': 'microsoft.github',
			'symbol': None,
			'message': None,
			'url': 'https://github.com/henryruhs/chroma-feedback/actions/runs/1',
			'status': 'passed'
		}
	]) == 'errored'


def test_resolve_report_status_warned() -> None:
	assert reporter.resolve_report_status(
	[
		{
			'name': 'microsoft.github',
			'symbol': None,
			'message': None,
			'url': 'https://github.com/henryruhs/chroma-feedback/actions/runs/1',
			'status': 'warned'
		},
		{
			'name': 'microsoft.github',
			'symbol': None,
			'message': None,
			'url': 'https://github.com/henryruhs/chroma-feedback/actions/runs/1',
			'status': 'errored'
		},
		{
			'name': 'microsoft.github',
			'symbol': None,
			'message': None,
			'url': 'https://github.com/henryruhs/chroma-feedback/actions/runs/1',
			'status': 'started'
		},
		{
			'name': 'microsoft.github',
			'symbol': None,
			'message': None,
			'url': 'https://github.com/henryruhs/chroma-feedback/actions/runs/1',
			'status': 'passed'
		}
	]) == 'warned'


def test_get_producer_status_failed() -> None:
	assert reporter.resolve_report_status(
	[
		{
			'name': 'microsoft.github',
			'symbol': None,
			'message': None,
			'url': 'https://github.com/henryruhs/chroma-feedback/actions/runs/1',
			'status': 'failed'
		},
		{
			'name': 'microsoft.github',
			'symbol': None,
			'message': None,
			'url': 'https://github.com/henryruhs/chroma-feedback/actions/runs/1',
			'status': 'warned'
		},
		{
			'name': 'microsoft.github',
			'symbol': None,
			'message': None,
			'url': 'https://github.com/henryruhs/chroma-feedback/actions/runs/1',
			'status': 'started'
		},
		{
			'name': 'microsoft.github',
			'symbol': None,
			'message': None,
			'url': 'https://github.com/henryruhs/chroma-feedback/actions/runs/1',
			'status': 'errored'
		},
		{
			'name': 'microsoft.github',
			'symbol': None,
			'message': None,
			'url': 'https://github.com/henryruhs/chroma-feedback/actions/runs/1',
			'status': 'passed'
		},
	]) == 'failed'


def test_get_producer_status_passed() -> None:
	assert reporter.resolve_report_status(
	[
		{
			'name': 'microsoft.github',
			'symbol': None,
			'message': None,
			'url': 'https://github.com/henryruhs/chroma-feedback/actions/runs/1',
			'status': 'passed'
		}
	]) == 'passed'



import atexit
import copy
from typing import Any, List

from chroma_feedback import color, helper, reporter
from chroma_feedback.typing import Color, Consumer, ProducerReport, Status
from .api import get_api

DEVICES = None


def get_devices() -> Any:
	global DEVICES

	if not DEVICES:
		DEVICES = get_api().devices
	return DEVICES


def filter_devices(devices : Any, device_serials : List[str]) -> Any:
	if device_serials:
		for device in copy.copy(devices):
			if device.serial not in device_serials:
				devices.remove(device)
	return devices


def process_devices(devices : Any, producer_report : List[ProducerReport]) -> List[Consumer]:
	result : List[Consumer] = []
	status : Status = reporter.resolve_report_status(producer_report)

	for device in devices:
		if set_device(device, color.get_by_status(status)):
			if helper.has_argument('-b') or helper.has_argument('--background-run'):
				register_reset_device(device)
			result.append(
			{
				'name': 'razer.chroma',
				'type': 'device',
				'description': helper.create_description(device.name, device.serial),
				'status': status
			})
	return result


def set_device(device : Any, color_config : Color) -> bool:
	if device.has('brightness'):
		device.brightness = color_config['brightness'][0]
	if device.fx.has('static'):
		return device.fx.static(*color_config['rgb'])
	return set_misc(device, color_config)


def set_misc(device : Any, color_config : Color) -> bool:
	state = False
	parts =\
	{
		'logo': 'logo',
		'scroll': 'scroll_wheel',
		'left': 'left',
		'right': 'right',
		'charging': 'charging',
		'fast_charging': 'fast_charging',
		'fully_charged': 'fully_charged',
		'backlight': 'backlight'
	}

	for part_key, part_value in parts.items():
		if device.fx.has(part_key + '_static'):
			function = getattr(getattr(device.fx.misc, part_value), 'static')
			state = function(*color_config['rgb']) or state
	return state


def register_reset_device(device : Any) -> None:
	atexit.register(lambda: set_device(device, color.get_reset()))



import atexit
import copy
from typing import Any, List

from chroma_feedback import color, helper, reporter
from chroma_feedback.typing import Color, Consumer, ProducerReport, Status
from .api import get_api

GROUPS = None


def get_groups(bridge_ip : str) -> Any:
	global GROUPS

	if not GROUPS:
		GROUPS = get_api(bridge_ip).get_group()
	return GROUPS


def filter_groups(groups : Any, group_names : List[str]) -> Any:
	if group_names:
		for index in copy.copy(groups):
			if groups[index]['name'] not in group_names:
				del groups[index]
	return groups


def process_groups(groups : Any, producer_report : List[ProducerReport]) -> List[Consumer]:
	result : List[Consumer] = []
	status : Status = reporter.resolve_report_status(producer_report)

	for index in groups:
		set_group(groups[index], color.get_by_status(status))
		if helper.has_argument('-b') or helper.has_argument('--background-run'):
			register_reset_group(groups[index])
		result.append(
		{
			'name': 'philips.hue',
			'type': 'group',
			'description': groups[index]['name'],
			'status': status
		})
	return result


def set_group(group : Any, color_config : Color) -> None:
	get_api(None).set_group(group['name'],
	{
		'hue': color_config['hue'],
		'sat': color_config['saturation'][1],
		'bri': color_config['brightness'][1],
		'on': True,
		'alert': 'none'
	})


def register_reset_group(group: Any) -> None:
	atexit.register(lambda: get_api(None).set_group(group['name'],
	{
		'on': False
	}))



from .core import process

ALL = \
{
	'agile_innovative.blinkstick': 'chroma_feedback.consumer.agile_innovative.blinkstick',
	'compulab.fit_statusb': 'chroma_feedback.consumer.compulab.fit_statusb',
	'elgato.streamdeck': 'chroma_feedback.consumer.elgato.streamdeck',
	'embrava.blynclight': 'chroma_feedback.consumer.embrava.blynclight',
	'embrava.blynclight_mini': 'chroma_feedback.consumer.embrava.blynclight_mini',
	'embrava.blynclight_plus': 'chroma_feedback.consumer.embrava.blynclight_plus',
	'gigabyte.rgb_fusion2': 'chroma_feedback.consumer.gigabyte.rgb_fusion2',
	'kuando.busylight_alpha': 'chroma_feedback.consumer.kuando.busylight_alpha',
	'kuando.busylight_omega': 'chroma_feedback.consumer.kuando.busylight_omega',
	'lifx': 'chroma_feedback.consumer.lifx.lifx',
	'luxafor.flag': 'chroma_feedback.consumer.luxafor.flag',
	'luxafor.mute': 'chroma_feedback.consumer.luxafor.mute',
	'luxafor.orb': 'chroma_feedback.consumer.luxafor.orb',
	'magic.hue': 'chroma_feedback.consumer.magic.hue',
	'muteme': 'chroma_feedback.consumer.muteme.muteme',
	'muteme.muteme_mini': 'chroma_feedback.consumer.muteme.muteme_mini',
	'mutesync': 'chroma_feedback.consumer.mutesync.mutesync',
	'nanoleaf': 'chroma_feedback.consumer.nanoleaf.nanoleaf',
	'philips.hue': 'chroma_feedback.consumer.philips.hue',
	'plantronics.status_indicator': 'chroma_feedback.consumer.plantronics.status_indicator',
	'razer.chroma': 'chroma_feedback.consumer.razer.chroma',
	'signify.wiz': 'chroma_feedback.consumer.signify.wiz',
	'thingm.blink1': 'chroma_feedback.consumer.thingm.blink1',
	'xiaomi.yeelight': 'chroma_feedback.consumer.xiaomi.yeelight'
}



from argparse import ArgumentParser
from typing import List

from chroma_feedback import helper, request
from chroma_feedback.typing import Producer
from .normalize import normalize_data

ARGS = None


def init(program : ArgumentParser) -> None:
	global ARGS

	if not ARGS:
		program.add_argument('--uptimerobot-host', default = 'https://api.uptimerobot.com')
		program.add_argument('--uptimerobot-slug', action = 'append')
		program.add_argument('--uptimerobot-token', required = True)
	ARGS = helper.get_first(program.parse_known_args())


def run() -> List[Producer]:
	result = []

	if ARGS.uptimerobot_slug:
		for slug in ARGS.uptimerobot_slug:
			result.extend(fetch(ARGS.uptimerobot_host, slug, ARGS.uptimerobot_token))
	else:
		result.extend(fetch(ARGS.uptimerobot_host, None, ARGS.uptimerobot_token))
	return result


def fetch(host : str, slug : str, token : str) -> List[Producer]:
	result = []
	response = None

	if host and slug and token:
		response = request.post(host + '/v2/getMonitors?api_key=' + token,
		{
			'search': slug
		})
	elif host and token:
		response = request.post(host + '/v2/getMonitors?api_key=' + token)

	if response and response.status_code == 200:
		data = request.parse_json(response)

		if 'monitors' in data:
			for monitor in data['monitors']:
				if 'friendly_name' in monitor and 'url' in monitor and 'status' in monitor:
					result.append(normalize_data(monitor['friendly_name'], monitor['url'], monitor['status']))
	return result



from argparse import ArgumentParser
from typing import Any, List

from chroma_feedback import helper, request
from chroma_feedback.typing import Producer
from .normalize import normalize_data

ARGS = None


def init(program : ArgumentParser) -> None:
	global ARGS

	if not ARGS:
		program.add_argument('--atlassian-bitbucket-host', default = 'https://api.bitbucket.org')
		program.add_argument('--atlassian-bitbucket-slug', action = 'append', required = True)
		program.add_argument('--atlassian-bitbucket-username', required = True)
		program.add_argument('--atlassian-bitbucket-password', required = True)
	ARGS = helper.get_first(program.parse_known_args())


def run() -> List[Producer]:
	result = []
	auth = fetch_auth(ARGS.atlassian_bitbucket_host, ARGS.atlassian_bitbucket_username, ARGS.atlassian_bitbucket_password)

	if 'access_token' in auth:
		for slug in ARGS.atlassian_bitbucket_slug:
			result.extend(fetch(ARGS.atlassian_bitbucket_host, slug, auth['access_token']))
	return result


def fetch(host : str, slug : str, token : str) -> List[Producer]:
	result = []
	response = None

	if host and slug and token:
		response = request.get(host + '/2.0/repositories/' + slug + '/pipelines/', headers = request.create_bearer_auth_headers(token))

	if response and response.status_code == 200:
		data = request.parse_json(response)

		if 'values' in data:
			build = helper.get_last(data['values'])

			if build and 'repository' in build and 'full_name' in build['repository'] and 'state' in build and 'name' in build['state']:
				if 'result' in build['state'] and 'name' in build['state']['result']:
					result.append(normalize_data(build['repository']['full_name'], build['state']['name'], build['state']['result']['name']))
				else:
					result.append(normalize_data(build['repository']['full_name'], build['state']['name'], None))
	return result


def fetch_auth(host : str, username : str, password : str) -> Any:
	result = {}
	response = None

	if host and username and password:
		response = request.post(host.replace('api.', '') + '/site/oauth2/access_token',
		{
			'grant_type': 'client_credentials'
		}, headers = request.create_basic_auth_headers(username, password))

	if response and response.status_code == 200:
		data = request.parse_json(response)

		if 'access_token' in data:
			result['access_token'] = data['access_token']
	return result



import sys
import webbrowser
from functools import partial
from typing import List

from PyQt6.QtCore import Qt
from PyQt6.QtGui import QAction, QBrush, QColor, QIcon, QPainter, QPixmap
from PyQt6.QtWidgets import QMenu, QSystemTrayIcon

from chroma_feedback import color, logger, loop, reporter, wording
from chroma_feedback.typing import ProducerReport, Status

SYSTRAY = None
MENU = None


def create(producer_report : List[ProducerReport]) -> None:
	global SYSTRAY, MENU

	if not SYSTRAY:
		SYSTRAY = QSystemTrayIcon()
	if not MENU:
		MENU = QMenu()
	update(producer_report)
	SYSTRAY.show()


def update(producer_report : List[ProducerReport]) -> None:
	global SYSTRAY, MENU

	status = reporter.resolve_report_status(producer_report)

	update_menu(producer_report)
	SYSTRAY.setContextMenu(MENU)
	SYSTRAY.setIcon(create_icon(status))


def is_created() -> bool:
	global SYSTRAY, MENU

	return SYSTRAY is not None and MENU is not None


def update_menu(producer_report : List[ProducerReport]) -> None:
	global MENU

	MENU.clear()

	for report in producer_report:
		item_report = MENU.addAction(report['message'])
		item_report.setIcon(create_icon(report['status']))
		item_report.setIconVisibleInMenu(True)
		if 'url' in report and report['url']:
			item_report.triggered.connect(partial(open_url, report['url']))
		else:
			item_report.setDisabled(True)
	if producer_report:
		MENU.addSeparator()

	item_start = MENU.addAction(wording.get('start'))
	item_stop = MENU.addAction(wording.get('stop'))
	item_exit = MENU.addAction(wording.get('exit'))
	item_start.setVisible(not loop.get_timer().isActive())
	item_stop.setVisible(loop.get_timer().isActive())
	item_start.triggered.connect(lambda: action_start(item_start, item_stop))
	item_stop.triggered.connect(lambda: action_stop(item_start, item_stop))
	item_exit.triggered.connect(action_exit)


def open_url(url : str) -> None:
	webbrowser.open(url)


def create_icon(status : Status) -> QIcon:
	color_config = color.get_by_status(status)
	pixmap = QPixmap(100, 100)
	pixmap.fill(Qt.GlobalColor.transparent)
	painter = QPainter(pixmap)
	painter.setBrush(QBrush(QColor(*color_config['rgb']), Qt.BrushStyle.SolidPattern))
	painter.drawEllipse(20, 20, 60, 60)
	painter.end()
	return QIcon(pixmap)


def action_start(item_start : QAction, item_stop : QAction) -> None:
	item_start.setVisible(False)
	item_stop.setVisible(True)
	loop.get_timer().start()


def action_stop(item_start : QAction, item_stop : QAction) -> None:
	item_start.setVisible(True)
	item_stop.setVisible(False)
	loop.get_timer().stop()


def action_exit() -> None:
	logger.info(wording.get('goodbye') + wording.get('exclamation_mark'))
	sys.exit()



from chroma_feedback import color, helper


def test_format_by_status() -> None:
	if helper.is_windows() is False:
		assert color.format_by_status('__test__', 'passed') == '\033[0;32m__test__\033[0m'
		assert color.format_by_status('__test__', 'started') == '\033[0;34m__test__\033[0m'
		assert color.format_by_status('__test__', 'errored') == '\033[0;37m__test__\033[0m'
		assert color.format_by_status('__test__', 'warned') == '\033[0;33m__test__\033[0m'
		assert color.format_by_status('__test__', 'failed') == '\033[0;31m__test__\033[0m'
	else:
		assert color.format_by_status('__test__', 'passed') == '__test__'
		assert color.format_by_status('__test__', 'started') == '__test__'
		assert color.format_by_status('__test__', 'errored') == '__test__'
		assert color.format_by_status('__test__', 'warned') == '__test__'
		assert color.format_by_status('__test__', 'failed') == '__test__'


def test_format_passed() -> None:
	if helper.is_windows() is False:
		assert color.format_passed('__test__') == '\033[0;32m__test__\033[0m'
	else:
		assert color.format_passed('__test__') == '__test__'


def test_format_started() -> None:
	if helper.is_windows() is False:
		assert color.format_started('__test__') == '\033[0;34m__test__\033[0m'
	else:
		assert color.format_passed('__test__') == '__test__'


def test_format_errored() -> None:
	if helper.is_windows() is False:
		assert color.format_errored('__test__') == '\033[0;37m__test__\033[0m'
	else:
		assert color.format_passed('__test__') == '__test__'


def test_format_warned() -> None:
	if helper.is_windows() is False:
		assert color.format_warned('__test__') == '\033[0;33m__test__\033[0m'
	else:
		assert color.format_passed('__test__') == '__test__'


def test_format_failed() -> None:
	if helper.is_windows() is False:
		assert color.format_failed('__test__') == '\033[0;31m__test__\033[0m'
	else:
		assert color.format_passed('__test__') == '__test__'


def test_get_by_status() -> None:
	assert color.get_by_status('passed') == color.get_passed()
	assert color.get_by_status('started') == color.get_started()
	assert color.get_by_status('errored') == color.get_errored()
	assert color.get_by_status('warned') == color.get_warned()
	assert color.get_by_status('failed') == color.get_failed()


def test_get_passed() -> None:
	state = color.get_passed()

	assert 'rgb' in state
	assert 'hue' in state
	assert 'saturation' in state
	assert 'brightness' in state
	assert 'kelvin' in state


def test_get_started() -> None:
	state = color.get_started()

	assert 'rgb' in state
	assert 'hue' in state
	assert 'saturation' in state
	assert 'brightness' in state
	assert 'kelvin' in state


def test_get_errored() -> None:
	state = color.get_errored()

	assert 'rgb' in state
	assert 'hue' in state
	assert 'saturation' in state
	assert 'brightness' in state
	assert 'kelvin' in state


def test_get_warned() -> None:
	state = color.get_warned()

	assert 'rgb' in state
	assert 'hue' in state
	assert 'saturation' in state
	assert 'brightness' in state
	assert 'kelvin' in state


def test_get_failed() -> None:
	state = color.get_failed()

	assert 'rgb' in state
	assert 'hue' in state
	assert 'saturation' in state
	assert 'brightness' in state
	assert 'kelvin' in state


def test_get_reset() -> None:
	state = color.get_reset()

	assert 'rgb' in state
	assert 'hue' in state
	assert 'saturation' in state
	assert 'brightness' in state
	assert 'kelvin' in state



import sys
from typing import Any

from chroma_feedback import logger, wording

API = None


def get_api() -> Any:
	global API

	if not API:
		API = api_factory()
	return API


def api_factory() -> Any:
	try:
		from liquidctl.driver.rgb_fusion2 import RgbFusion2 as api

		return api
	except ImportError:
		logger.error(wording.get('package_not_found').format('liquidctl') + wording.get('exclamation_mark'))
		sys.exit()



#!/usr/bin/env python3

from setuptools import setup
from chroma_feedback import metadata
from chroma_feedback.install import InstallCommand

setup(
	name = metadata.get('name'),
	description = metadata.get('description'),
	long_description = open('README.md').read(),
	long_description_content_type = 'text/markdown',
	version = metadata.get('version'),
	license = metadata.get('license'),
	keywords = metadata.get('keywords'),
	author = metadata.get('author'),
	author_email = metadata.get('author_email'),
	url = metadata.get('url'),
	packages =
	[
		'chroma_feedback',
		'chroma_feedback.producer',
		'chroma_feedback.producer.appveyor.appveyor',
		'chroma_feedback.producer.atlassian.bamboo',
		'chroma_feedback.producer.atlassian.bitbucket',
		'chroma_feedback.producer.buddy.buddy',
		'chroma_feedback.producer.circle.circle',
		'chroma_feedback.producer.cloudbees.codeship',
		'chroma_feedback.producer.custom.custom',
		'chroma_feedback.producer.datadog.datadog',
		'chroma_feedback.producer.gitlab.gitlab',
		'chroma_feedback.producer.heroku.heroku',
		'chroma_feedback.producer.jenkins.jenkins',
		'chroma_feedback.producer.jetbrains.teamcity',
		'chroma_feedback.producer.microsoft.azure',
		'chroma_feedback.producer.microsoft.github',
		'chroma_feedback.producer.netlify.netlify',
		'chroma_feedback.producer.travis.travis',
		'chroma_feedback.producer.uptimerobot.uptimerobot',
		'chroma_feedback.producer.vercel.vercel',
		'chroma_feedback.consumer',
		'chroma_feedback.consumer.agile_innovative.blinkstick',
		'chroma_feedback.consumer.compulab.fit_statusb',
		'chroma_feedback.consumer.elgato.streamdeck',
		'chroma_feedback.consumer.embrava.blynclight',
		'chroma_feedback.consumer.embrava.blynclight_mini',
		'chroma_feedback.consumer.embrava.blynclight_plus',
		'chroma_feedback.consumer.gigabyte.rgb_fusion2',
		'chroma_feedback.consumer.kuando.busylight_alpha',
		'chroma_feedback.consumer.kuando.busylight_omega',
		'chroma_feedback.consumer.lifx.lifx',
		'chroma_feedback.consumer.luxafor.flag',
		'chroma_feedback.consumer.luxafor.mute',
		'chroma_feedback.consumer.luxafor.orb',
		'chroma_feedback.consumer.magic.hue',
		'chroma_feedback.consumer.muteme.muteme',
		'chroma_feedback.consumer.muteme.muteme_mini',
		'chroma_feedback.consumer.mutesync.mutesync',
		'chroma_feedback.consumer.nanoleaf.nanoleaf',
		'chroma_feedback.consumer.philips.hue',
		'chroma_feedback.consumer.plantronics.status_indicator',
		'chroma_feedback.consumer.razer.chroma',
		'chroma_feedback.consumer.signify.wiz',
		'chroma_feedback.consumer.thingm.blink1',
		'chroma_feedback.consumer.xiaomi.yeelight'
	],
	cmdclass =
	{
		'install': InstallCommand
	},
	entry_points =
	{
		'console_scripts':
		[
			'chroma-feedback = chroma_feedback.core:cli',
		]
	},
	install_requires =
	[
		'busylight-for-humans==0.32.0',
		'lifxlan==1.2.8',
		'liquidctl==1.13.0',
		'nanoleafapi==2.1.2',
		'phue==1.1',
		'python-magichue==0.3.2',
		'pyqt6==6.7.1 ',
		'pywizlight==0.5.14',
		'requests==2.32.3 ',
		'streamdeck==0.9.6',
		'yeelight==0.7.14'
	],
	tests_require =
	[
		'flake8==6.1.0',
		'mock==5.1.0',
		'mypy==1.5.1',
		'pytest==7.4.0',
		'pytest-cov==4.1.0',
		'pytest-dotenv==0.5.2',
		'pytest-mock==3.11.1',
		'types-mock==5.1.0.1',
		'types-requests==2.31.0.2'
	]
)
